##############################################
# Copyright (R): Zhiping Liu and Danning He
##############################################
#step 1: identify subnetworks
#input:  "chd.txt"
#        "df.txt"
#        "weighted_ppi.txt"
#
#output: "subnetwork.txt"
##############################################
library(igraph)

#the edge attribute: distance=1-|cor(x,y)|
dat=read.table("weighted_ppi.txt",header=TRUE)
ppi=graph.data.frame(dat,directed=FALSE)
save(ppi,file="weighted_ppi.RData")

chd=readLines("chd.txt")
chd=intersect(V(ppi)$name,chd)
df=readLines("df.txt")
df=intersect(V(ppi)$name,df)

#distance function returns the sum of distance of a path
distance=function(path){
sum(E(ppi)$weight[E(ppi) %in% E(ppi,path=path)])
}

#choose function returns the path with minimum sum of distance
choose=function(list){
result=unlist(lapply(list,distance))
list[[which.min(result)]]
}

return_index=function(graph,id){
nodes=V(graph)$name
ind=0:(length(nodes)-1)
ind[nodes %in% id]
}

return_id=function(graph,index){
nodes=V(graph)$name
ind=0:(length(nodes)-1)
nodes[ind %in% index]
}

#get_subnetwork returns a list in which each element 
#contains the nodes in a subnetwork. 
get_subnetwork=function(chd,df,ppi){
cluster=list()
from=return_index(ppi,chd)
to=return_index(ppi,df)
iteration=length(from)
for(i in 1:(iteration)){
list=list()
for(j in 1:length(to)){
k=1
tmp=get.all.shortest.paths(ppi,from[i],to[j])
if(length(tmp)>0){
list[[k]]=choose(tmp)
k=k+1
}
}
cluster[[i]]=factor(return_ID(unique(unlist(list))))
}
cluster
}

#write subnetwork to a file("subnetwork.txt") in which COL1 corresponds to the
#name of subnetwork, and COL2 corresponds to the nodes in the subnetwork.
write_subnetwork=function(cluster,chd){
tmp=list()
mem=unlist(lapply(cluster,length))
for(i in 1:length(chd)){
tmp[[i]]=rep(chd[i],mem[i])
}
member=cbind(data.frame(unlist(tmp)),data.frame(unlist(cluster)))
write.table(member,"subnetwork.txt",header=FALSE,row.names=FALSE,quote=FALSE)
}

cluster=get_subnetwork(chd,df,ppi)
write_subnetwork(cluster,chd)

#################################################
#step 2: compute information flow for each nodes
#in each subnetwork.
#
#calculation of information flow is based on MATLAB 
#source code provided by:
#Missiuro PV, Liu K, Zou L, Ross BC, Zhao G, Liu JS, 
#Ge H: Information Flow Analysis of Interactome Networks. 
#PLoS Comput Biol 2009, 5(4):e1000350.
#
#please see guidelines in current_info_flow.mat and
#info_module.mat
#
#below takes the output of info_module.mat:
#"info_current.txt" and produces "subnetwork_current.txt"
#containing information flow for each nodes in each
#subnetwork.
#################################################

%beginning of info_module.mat
load info_input.txt
load weight_array.txt
load df.txt
mat=spconvert(weight_array);
mat=mat+mat.';
list=info_input;
chd=unique(list(:,1));

for i=1:length(chd);
    node=chd(i);
    module=list(list(:,1)==node,2);
    [c,ia_chd,ib]=intersect(module,chd);
    [c,ia_df,ib]=intersect(module,df);
    B=mat(module,module);
    [nodesI,nodesAverageI] = current_info_flow(B,1,ia_chd,ia_df);
    info{i}=[repmat(node,length(module),1) module nodesI];
end;
final=cat(1,info{:});
dlmwrite('info_current.txt',final,' ')
%end of info_module.mat


%beginning of current_info_flow.mat
function [nodesI,nodesAverageI] = current_info_flow(connectMatrix, condOrRes)
% current_info_flow.m
%
% Created by Patrycja Missiuro {patrycja@mit.edu}, Whitehead Institute, 
% Date: March 2006
% Copyright: Massachusetts Institute of Technology / Whitehead Institute,
% Cambridge, MA
%
% Inputs:
%   connectMatrix: connectivity matrix of all the nodes, where at v(i,j) =
%       v(j,i) position and has distance between nodes i and j, size N by N 
%       all positions in the matrix must be >= 0, also, all nodes need to be
%       interconnected (there exists a path from any one to any other node)
%   condOrRes: conductance or resistance, default cond, equal to 1
% Outputs:
%   nodesI: current through every node, size N by 1
%
% stamp the matrix of Resistors (only needs to be done once)
% matrix of resistaors, size same as connectMatrix

if ~exist('condOrRes','var')
    condOrRes = 1;
end;

N = size(connectMatrix,1);
A = sparse(zeros(N,N));
nodesI = zeros(N,1);
% Step 1: fill in the matrix of resistors, A
% Also, set up matrix R2 needed for Step 6, also sparse
% it is used to solve for the branch (NOT node) currents (eg. between nodes
% i and j), size M by N where N is number of nodes, M is number of branches
R2 = sparse(0,N);
for i=1:N
    indxConnNodes = find(connectMatrix(i,:) ~= 0);
    % add their values up
    if (condOrRes == 1) % conductance
        A(i,i) = - sum(connectMatrix(i,indxConnNodes));
    else % resistance
        A(i,i) = - sum(1 / connectMatrix(i,indxConnNodes));
    end;
    % fill in the off diagonals, only the one for the row we are working
    % with (the upper one)
    if (condOrRes == 1) % conductance
        A(i,indxConnNodes) = connectMatrix(i,indxConnNodes);
    else % resistance
        A(i,indxConnNodes) = 1 / connectMatrix(i,indxConnNodes);
    end;
    
    % setup R2 (used later on)
    % for each of the branches, we need to put a row into the matrix with 
    % conductance/resistance on it
    nodesForR2 = indxConnNodes(indxConnNodes > i);
    pieceToAppend = zeros(length(nodesForR2),N);
    if (condOrRes == 1)
        pieceToAppend(:,i) = connectMatrix(i,nodesForR2)';
        for k=1:length(nodesForR2)
            pieceToAppend(k,nodesForR2(k)) = - connectMatrix(i,nodesForR2(k));
        end;
    else
        pieceToAppend(:,i) = 1 / connectMatrix(i,nodesForR2)';
        for k=1:length(nodesForR2)
            pieceToAppend(k,nodesForR2(k)) = - 1/ connectMatrix(i,nodesForR2(k));
        end;
    end;
    % append the piece to R2
    R2 = [R2 ;pieceToAppend];
end;
% matrix A and R2 only need to be set up once, done
% also form R2T (transpose of R2, with 1s where nonzero entries are), used
% in step 7

R2T = sparse((R2 ~= 0)');

% Step 2: Iterate over the ground node i_g=1:N, force the ground node to 0
% by setting the appropriate row of A to all zeros except 1 at the i_g
% position.
count = 0;
for ig=1:N
    % make a copy of A matrix, and force one node to ground
    Ag = sparse(A);
    Ag(ig,:) = zeros(1,N);
    Ag(ig,ig) = 1;
    % this Ag matrix stays the same for the next loop, thus we can do LU
    % decomposition on it once per ground node
    
    % Step 3: LU decomposition of matrix Ag
    [L,U,P, Q] = lu(Ag);
    
    % Step 4: within the ground node loop, iterate over the source node, and set the
    % RHS to 1 for that node

    % i_vec is net vector of currents into the node,
    % we have A * phi = i_vec, where phi is a vector to solve for,
    % phi is a vector of node voltages
    i_vec = zeros(N,1);
    for is = ig+1:N % 
        i_vec(is) = 1; % set this value to the current source of 1 Amp
        
        % Step 5: solve for phi - node voltages
        phi = Q*(U\  (L\  (P * i_vec)));
      
        % Step 6: Solve for current value flowing through each node        
        % R2 * phi = i_branch (i_branch is the current in each branch, size
        % m by 1, m is number of branches
        i_branch = R2 * phi;
        % Step 7: Solve for all the currents in nodes (what we want)
        i_nodes = R2T * abs(i_branch)/2;
        % set the contribution of the end nodes to 0
        i_nodes([is ig]) = 0;
        nodesI = nodesI + i_nodes;
        
        count = count + 1;
        % reset the vector to zeros
        i_vec(is) = 0;
    end;
        
end;

if (count ~= 0)
    nodesAverageI = nodesI / count;
else 
    nodesAverageI = 0;
end;
%end of current_info_flow.mat


current=read.table("info_current.txt")
chd=unique(current$V1)
member=unique(current$V2)
gl=list()
summary=data.frame(matrix(ncol=4,nrow=length(member)))
belong=data.frame(matrix(nrow=length(member),ncol=3))
names(summary)=c("count","mean","median","variance")
for(i in 1:length(member)){
gl[[i]]=current$V3[current$V2 %in% member[i]]
tmp=current[current$V2 %in% member[i],]
remain=tmp[which.max(tmp$V3),]
belong[i,]=remain[1,]
summary[i,]=c(length(gl[[i]]),mean(gl[[i]]),median(gl[[i]]),var(gl[[i]]))
}
names(gl)=member
o=order(belong$X1)
write.table(belong[o,],"subnetwork_current.txt",
row.names=FALSE,quote=FALSE)

##############################
#step 3: identify modules
##############################
nodes=factor(unique(unlist(cluster)))
subnetwork=subgraph(ppi,return_index(ppi,nodes))
save(subnetwork,file="subnetwork.RData")
current=read.table("subnetwork_current.txt",header=TRUE)
current$module=as.character(current$module)
current$gene=as.character(current$gene)

#check function returns the unconnected nodes from each modules.
check=function(nodes,ppi){
subgraph=subgraph(ppi,return_index(ppi,nodes))
if(is.connected(subgraph)==FALSE){
result=clusters(subgraph)
index=0:(result$no-1)
remain=index[which.max(result$csize)]
if(max(result$csize)>5){
output=V(subgraph)$name[result$membership!=remain]
}else{
output=V(subgraph)$name
}
output
}
}

#get_module function returns the module
get_module=function(current,subnetwork){
module=unique(current$module)
member=unique(current$gene)
mem_ind=1:length(member)
gl=list()
belong=current[1:length(member),]
names(belong)=names(current)
belong=current[1:length(member),]
for(i in 1:length(member)){
tmp=current[current$gene %in% member[i],]
tmp=tmp[order(tmp$current,decreasing=TRUE),]
gl[[i]]=tmp$current
names(gl[[i]])=tmp$module
gl[[i]]=gl[[i]][order(gl[[i]],decreasing=TRUE)]
remain=tmp[which.max(tmp$current),]
belong[i,]=tmp[which.max(tmp$current),]
}
names(gl)=member
belong2=belong
count=1
while(TRUE){
check_genes=list()
for(i in 1:length(module)){
genes=belong$gene[belong$module==module[i]]
if(length(genes)>0){
check_genes[[i]]=check(genes,sub)
}
}
change=unlist(check_genes)
if(length(change)>0){
count=count+1
for(i in 1:length(change)){
index=belong$gene==change[i]
belong[index,3]=gl[[mem_ind[member==change[i]]]][count]
belong[index,1]=names(gl[[mem_ind[member==change[i]]]])[count]
}
}else{
break
}
}
belong
}

module=get_module(current,subnetwork)
write.table(module,"module.txt",quote=FALSE,row.names=FALSE)

